@@ -2,39 +2,12 @@ import numpy as np
 from numpy import trapz
 
 from scipy.interpolate import interp1d
+from scipy.signal import find_peaks
 from hysteresis import data
-
-
 import hysteresis.env as env
 import matplotlib.pyplot as plt
 
 
-"""
-TODO:
-    For Monotonic curves:
-    Find and time between peaks is optional.
-    Find interesections
-    FInd area nearest to current peak
-"""
-
-
-"""
-TODO:
-    Allow for custom headings:
-    Perhaps a style object?
-
-    Add limitts to the style object?    
-    Make the limits part of the hysteresis object, so they don't need
-    to continually be passed to each funciton.
-"""
-
-
-
-"""
-TODO: Make Plot functions Lamda function, then specialize for slope, area, etc.
-
-"""
-
 
 # =============================================================================
 # Curve objects
@@ -59,13 +32,13 @@ class CurveBase:
         ----------
         XYData : array
             The input array of XY date for the curve.
-        AreaFunction : function, optional
+        areaFunction : function, optional
             The function to be used to calcualte area. 
-            The default is defaultAreaFunction.
-        slopefunction : function, optional
+            The default is defaultareaFunction.
+        slopeFunction : function, optional
             The function to be used to calcualte slope. 
-            The default is defaultAreaFunction.
-        plotfunction : function, optional
+            The default is defaultareaFunction.
+        plotFunction : function, optional
             The function to be used to plot the curve. 
             The default is defaultPlotFunction.
         xunit : str, optional
@@ -76,16 +49,13 @@ class CurveBase:
         """
         self.xy = XYData
         self.Npoints = len(XYData[:,0])
-        # self.AreaFunction = fArea
-        # self.slopefunction = fslope
-        # self.plotfunction = fplot
         
-        self.AreaFunction = env.environment.fArea
-        self.slopefunction = env.environment.fslope
-        self.plotfunction = env.environment.fplot
+        self.areaFunction = env.environment.fArea
+        self.slopeFunction = env.environment.fslope
+        self.lengthFunction = env.environment.flength
+        self.plotFunction = env.environment.fplot
         
         self.initializeFig = env.environment.finit
-        self.plotfunction = env.environment.fplot
         self.showCycles = env.environment.fcycles
         
         self.colorDict = {0:'C0', 1:'C1', 2:'C3'}
@@ -116,8 +86,8 @@ class CurveBase:
         
     def setArea(self):
         """ sets the area under each point of the curve using the area function"""
-        self.Area = self.AreaFunction(self.xy)
-        return self.Area
+        self.area = self.areaFunction(self.xy)
+        return self.area
     
     def getCumDisp(self):
         """ Gets the absolute value of cumulative displacement of the curve at each x value.
@@ -125,36 +95,36 @@ class CurveBase:
         dx = np.diff(self.xy[:,0])
         return np.append(0, np.cumsum(np.abs(dx)))
     
-    def getNetCumDisp(self, StartIndex = 0, EndIndex = 0):
+    def getNetCumDisp(self, startIndex = 0, endIndex = 0):
         """ Gets the total cumulative displacement between the start and start and end 
         indexes. By default the whole curve us used.
         """
         x = self.xy[:,0]
         dx = np.append(0, np.diff(self.xy[:,0]))
         
-        if EndIndex == 0:
-            EndIndex = self.Npoints
+        if endIndex == 0:
+            endIndex = self.Npoints
                    
-        return np.sum(np.abs(dx[StartIndex:EndIndex]))
+        return np.sum(np.abs(dx[startIndex:endIndex]))
     
     def getCumArea(self):
         """ Gets the cumulative area under the curve for the entire curve
         """
-        Area = self.Area
+        Area = self.area
         return np.cumsum(Area)
     
-    def getNetArea(self, StartIndex = 0, EndIndex = 0):
+    def getNetArea(self, startIndex = 0, endIndex = 0):
         """
         Returns the net area between two indexes in the xy curve. The default 
         setting is to return the area for the whole cuve.
 
         """
-        Area = self.Area
+        Area = self.area
 
-        if EndIndex == 0:
-            EndIndex = self.Npoints
+        if endIndex == 0:
+            endIndex = self.Npoints
                    
-        return np.sum(Area[StartIndex:EndIndex])
+        return np.sum(Area[startIndex:endIndex])
            
     def setSlope(self):
         """
@@ -164,15 +134,26 @@ class CurveBase:
         
         # Calculate end point slope
         xy = self.xy
-        self.Slope  = self.slopefunction(xy)
-    
+        self.slope  = self.slopeFunction(xy)
+
+    def setLength(self):
+        """
+        Calcuates the slope of the curve at each point.
+        The user can pass in a custom function that calculates the slope.
+        """
+        
+        # Calculate end point slope
+        xy = self.xy
+        self.length  = self.lengthFunction(xy)
+
+
     def setPeaks(self, peakDist = 2, peakWidth = None, peakProminence = None):
         """
         Finds the indexes of max and min points, then stores them.
         """
         
         y = self.xy[:,1]
-        peakIndexes = data.GetCycleIndicies(y, peakDist, peakWidth, peakProminence)        
+        peakIndexes = data.getCycleIndicies(y, peakDist, peakWidth, peakProminence)        
         self.peakIndexes = peakIndexes
         
         xy = self.xy
@@ -190,7 +171,7 @@ class CurveBase:
         x = self.xy[:,0]
         y = self.xy[:,1]
                     
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
+        return self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
                 
     def plotVsIndex(self, plotCycles = False, plotPeaks = False, 
                      labelCycles = []):
@@ -201,11 +182,11 @@ class CurveBase:
         x = np.arange(0,len(self.xy[:,0]))
         y = self.xy[:,0]
                     
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
+        self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
 
     def plotLoadProtocol(self, comparisonProtocol = []):
         """
-        Plots the load protcol of the curve.
+        Plots the peak x values for each cycle in acurve.
         """           
         plotCycles = False
         plotPeaks = False
@@ -213,25 +194,26 @@ class CurveBase:
         y = self.loadProtocol
         x = np.arange(0,len(y))
                     
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
+        self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
         
         if len(comparisonProtocol) != 0:
             plt.plot(comparisonProtocol)    
     
+    
     def plotSlope(self,  plotCycles = False, plotPeaks = False, 
                   labelCycles = []):
         
         x = self.xy[:,0]
-        y = self.Slope
+        y = self.slope
 
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
+        self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)
                 
     def plotArea(self,  plotCycles = False, plotPeaks = False, labelCycles = []):
         
         x = self.xy[:,0]
-        y = self.Area
+        y = self.area
 
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)  
+        self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)  
                         
     def plotCumArea(self,  plotCycles = False, plotPeaks = False, labelCycles = []):
         
@@ -239,7 +221,7 @@ class CurveBase:
         x = self.getCumDisp()
         y = self.getCumArea()
 
-        self.plotfunction(self, x ,y, plotCycles, plotPeaks, labelCycles)  
+        self.plotFunction(self, x ,y, plotCycles, plotPeaks, labelCycles)  
              
     def initFig(self, xlims = [], ylims = []):
         return self.initializeFig(xlims, ylims)
@@ -310,7 +292,7 @@ class Hysteresis(CurveBase):
         self.setReversalPropreties(revDist, revWidth, revProminence)
         
         x = self.xy[:,0]
-        self.reversalIndexes = data.GetCycleIndicies(x, revDist, revWidth, revProminence)
+        self.reversalIndexes = data.getCycleIndicies(x, revDist, revWidth, revProminence)
         self.loadProtocol = x[self.reversalIndexes]
 
     def getReversalxy(self):
@@ -331,25 +313,25 @@ class Hysteresis(CurveBase):
             # I forget why we overshoot the cycles here by one
             Cycles[ii] = SimpleCycle(xy[indices[ii]:(indices[ii+1]+1), :])
                        
-        self.Cycles = Cycles
+        self.cycles = Cycles
         self.NCycles = NIndex
         self.NCycles = len(Cycles)
     
     def getCycles(self, Indicies):
         """ Returns a list of cycles given a list of indicies"""
-        Cycles = [self.Cycles[index] for index in Indicies]
+        Cycles = [self.cycles[index] for index in Indicies]
         return Cycles      
     
     def getCycle(self, Index):
         """ Returns a list of cycles given a input index"""
-        return self.Cycles[Index]
+        return self.cycles[Index]
 
     def setCycleNetAreas(self):
         """ Calculates the net area under each cycle."""
         areas = np.zeros(self.NCycles)
-        for ii, vector in enumerate(self.Cycles):
+        for ii, vector in enumerate(self.cycles):
             # Skip the last point to avoid counting it twice!
-            areas[ii] = vector.getNetArea(EndIndex = -1)
+            areas[ii] = vector.getNetArea(endIndex = -1)
             
         self.CycleAreas = areas
 
@@ -359,14 +341,14 @@ class Hysteresis(CurveBase):
 
     def plotCycle(self, Index, plotPeaks = False):
         """ Plots a specific cycle."""
-        Cycle = self.Cycles[Index]
+        Cycle = self.cycles[Index]
         return Cycle.plot(plotPeaks = plotPeaks)
 
     def plotCycles(self, Cycles = [], plotCycles = False, plotPeaks = False, 
                     labelCycles = []):
         """ Plots several cycles on the same figure """
         xyHys = self.xy
-        Vectors = self.Cycles
+        Vectors = self.cycles
         # fig, ax = initializeFig(xlim, ylim)
         
         self.showCycles(self, xyHys[:,0], xyHys[:,1], plotCycles, plotPeaks, labelCycles, Cycles)
@@ -432,7 +414,7 @@ class Hysteresis(CurveBase):
         
     def recalculateCycles_like(self, sampleHysteresis):
         """
-        Recalulates the peaks of one hysteresis using another the reversal
+        Recalulates the cycles of one hysteresis using another the reversal
         propreties from another hysteresis. Peaks are calculated using scipy's 
         find_peaks function.
         
@@ -456,7 +438,49 @@ class Hysteresis(CurveBase):
         self.setReversalIndexes(revDist, revWidth, revProminence)
         self.setCycles()
         self.setArea()        
-    
+        
+    def recalculateCycles_dist(self, revDist = 2, revWidth = None, 
+                               revProminence = None, **kwargs):
+        """
+        Recalulates the reversals of one hysteresis using another the reversal
+        propreties from another hysteresis. Peaks are calculated using scipy's 
+        find_peaks function.
+        
+        Parameters
+        ----------
+             
+        revDist : number, optional
+            The minimum minimal numbr of indexese (>= 1) in samples between
+            neighbouring peaks. Smaller peaks are removed first until the condition
+            is fulfilled for all remaining peaks.
+        revProminence : number or ndarray or sequence, optional
+            Required prominence of peaks. Either a number, ``None``, an array
+            matching `x` or a 2-element sequence of the former. The first
+            element is always interpreted as the  minimal and the second, if
+            supplied, as the maximal required prominence.
+        revWidth : number or ndarray or sequence, optional
+            Required width of peaks in samples. Either a number, ``None``, an array
+            matching `x` or a 2-element sequence of the former. The first
+            element is always interpreted as the  minimal and the second, if
+            supplied, as the maximal required width.
+        
+        Returns
+        -------
+        None.
+
+        """        
+        
+        
+        try:
+            dxy = self.length
+        except:
+            raise Exception('Length not set yet, set length using .setLength')
+                        
+        self.reversalIndexes = data.getMaxIndicies(dxy, revDist, revWidth, revProminence)
+        self.loadProtocol = self.xy[self.reversalIndexes,0]
+        
+        self.setCycles()
+        self.setArea()           
 
     def RemoveCycles():
         pass
@@ -511,7 +535,7 @@ class SimpleCycle(CurveBase):
         for ii in range(NIndex):
             SubCycles[ii] = MonotonicCurve(xy[indices[ii]:(indices[ii+1]+1), :])
                        
-        self.SubCycles = SubCycles
+        self.subCycles = SubCycles
         self.NsubCycles = len(SubCycles)
         
         
@@ -520,7 +544,7 @@ class SimpleCycle(CurveBase):
         Sets the net area for all SubCycles
         """
         try:
-            SubCycles = self.SubCycles
+            SubCycles = self.subCycles
         except:
             raise Exception('No SubCycles not yet set')
             
@@ -532,7 +556,7 @@ class SimpleCycle(CurveBase):
         Sets the net area for all MonotonicCurves
         """
         try:
-            SubCycles = self.SubCycles
+            SubCycles = self.subCycles
         except:
             raise Exception('SubCycles not yet set')
             
@@ -540,16 +564,16 @@ class SimpleCycle(CurveBase):
             SubCycle.setSlope()       
       
     def getSubCycles(self, Indicies):
-        SubCycles = [self.SubCycles[index] for index in Indicies]
+        SubCycles = [self.subCycles[index] for index in Indicies]
         return SubCycles      
     
     def getSubCycle(self, Index):
-        return self.SubCycles[Index]
+        return self.subCycles[Index]
      
     def plotSubCycles(self, SubCyclesIndicies = [], plotCycles = False, plotPeaks = False):
         
         xyMono = self.xy
-        Vectors = self.SubCycles
+        Vectors = self.subCycles
         
         self.showCycles(self, xyMono[:,0], xyMono[:,1], plotCycles, plotPeaks)
         
@@ -627,4 +651,31 @@ def findPeakKwargs(**kwargs):
     if 'peakProminence' in kwargs.keys():
         print('Use of "peakProminence" has been depricated. Instead use "revProminence".')    
     
-    # pass
\ No newline at end of file
+    # pass
+
+
+"""
+TODO:
+    For Monotonic curves:
+    Find and time between peaks is optional.
+    Find interesections
+    FInd area nearest to current peak
+"""
+
+
+"""
+TODO:
+    Allow for custom headings:
+    Perhaps a style object?
+
+    Add limitts to the style object?    
+    Make the limits part of the hysteresis object, so they don't need
+    to continually be passed to each funciton.
+"""
+
+
+
+"""
+TODO: Make Plot functions Lamda function, then specialize for slope, area, etc.
+
+"""
